
#include "korners_game_engine_ai.h"

#include <QPoint>
#include <KDebug>


const char KornersGameEngineAI::static_values_end[] = { 0,  1,  1,  2,  2,  2,  2,  2,
                                                        1,  1,  2,  3,  3,  3,  3,  3,
                                                        1,  2,  3,  4,  4,  4,  4,  4,
                                                        2,  3,  6,  6,  6,  6,  5,  5,
                                                        2,  3,  6,  8,  8,  8,  7,  7,
                                                        2,  3,  5,  7,  10, 11, 12, 13,
                                                        2,  3,  5,  7,  11, 12, 13, 14,
                                                        2,  3,  5,  7,  12, 13, 14, 14  };

const char KornersGameEngineAI::static_values_begin[] = { 0,  1,  1,  2,  2,  2,  2,  2,
                                                          1,  1,  2,  3,  3,  3,  3,  3,
                                                          1,  2,  3,  4,  4,  4,  4,  4,
                                                          2,  3,  6,  6,  6,  6,  5,  5,
                                                          2,  3,  6,  8,  8,  8,  7,  7,
                                                          2,  3,  5,  7,  10, 10, 10, 11,
                                                          2,  3,  5,  7,  10, 11, 11, 11,
                                                          2,  3,  5,  7,  14, 11, 11, 11  };


KornersGameEngineAI::KornersGameEngineAI() 
{
    for(int i = 0; i < N_CAPS/2; i++)
    {
        m_capPositions.push_back( QPoint( i%IN_ROW, i/IN_ROW ) );
    }

    m_endSpiel = false;
}

int KornersGameEngineAI::getScores( std::multimap<int, int>& all,
                                    std::list<MoveInfo>& ret,
                                    owner tp )
{
    int old_pos_value = calcPosStaticValue( tp );
    QPoint mid = calcMidPosition();

    ret.clear();

    std::multimap<int, int>::iterator it;

    int value = 0;
    int n_good_moves = 0;

    for( it = all.begin(); it != all.end(); ++it )
    {
        applyMove( it->first, it->second );
        value = calcPosStaticValue( tp );
        if( value > old_pos_value )
                n_good_moves++;

        // several additional considerations for opponent
        if( tp == opponent )
        {
            value += (value - old_pos_value)*2;
            value += moveValue( indexToPoint(it->first), 
                                indexToPoint(it->second) );
        }
        
        value += backwardPenalty( indexToPoint(it->first), mid );

        ret.push_back( MoveInfo( indexToPoint(it->first), indexToPoint(it->second), value ) );
        applyMove( it->second, it->first );
    }

    ( n_good_moves < 4 )? n_good_moves = 4 : n_good_moves;

    ret.sort( std::greater<MoveInfo>( ) );
    return n_good_moves;
}

int KornersGameEngineAI::scoreAvailableMoves( std::list<MoveInfo>& moves, owner tp)
{
    bool reachable[ KornersEngine::FIELD_SIZE*KornersEngine::FIELD_SIZE ];
    std::multimap<int, int> srcVsDests;
    QVector<int> avail_moves;

    for( int i = 0; i < m_capPositions.size(); i++ )
    {
        QPoint src = m_capPositions.at(i);
        memset( reachable, false, FIELD_SIZE*FIELD_SIZE );
        avail_moves.clear();
        markReachable( src, none, reachable );
        pointsToGo( reachable, avail_moves );
        for( int j = 0; j < avail_moves.size(); j++ )
            srcVsDests.insert( std::make_pair<int, int>( pointToIndex(src), avail_moves[j] ) );
    }

    return getScores( srcVsDests, moves, tp );
}

KornersGameEngineAI::MoveInfo KornersGameEngineAI::search( int depth )
{
    std::list<MoveInfo> moves;

    m_curPositionVal = calcPosStaticValue( opponent );

    if( calcEnemyBaseCaps( opponent ) > 9 )
        m_endSpiel = true;

    int n_good_moves = scoreAvailableMoves( moves, opponent);

    ( n_good_moves > 9 ) ? n_good_moves = 9 : n_good_moves;

    std::list<MoveInfo>::iterator it;
    int i = 0;
    int diff = 0;

    for( it = moves.begin() ; it != moves.end() ; i++, ++it )
    {
        if( i == n_good_moves )
            break;          
        
        makeMove( (*it).begin, (*it).end );

        if( innerSearch( &diff, depth - 1, player )  ) 
        {
            makeMove( (*it).end, (*it).begin );
            return *it;
        }

        (*it).profit = diff;
        diff = 0;
        makeMove( (*it).end, (*it).begin );
    }

    moves.sort( std::greater<MoveInfo>() );
    return ( moves.front() );
}

bool KornersGameEngineAI::innerSearch( int* diff, int depth, owner tp )
{
    if( gameOver() == opponent )
    {
        *diff += 1000;
        return true;
    }

    bool ret = false;

    if( depth%2 == 0 )
    {
        int playerVal = calcPosStaticValue(player);     
        int opponentVal = calcPosStaticValue(opponent);         
            
        if( ( opponentVal > playerVal ) && ( opponentVal > m_curPositionVal ) )
        {
            int this_diff = opponentVal + ( opponentVal - m_curPositionVal )*2; 

            *diff += this_diff;
            return true; 
        }
        
        if( depth == 0 )
        {
            int this_diff = opponentVal + ( opponentVal - m_curPositionVal )*2;

            *diff += this_diff;
            return false; 
        }
    }

    std::list<MoveInfo> moves;
    int n_good_moves = scoreAvailableMoves( moves, tp);

    ( n_good_moves > 5 ) ? n_good_moves = 5 : n_good_moves;

    std::list<MoveInfo>::iterator it;
    int i = 0;
    for( it = moves.begin() ; it != moves.end() ; i++, ++it )
    {
        if( i == n_good_moves )
                break;

        makeMove( (*it).begin, (*it).end );
        ret = innerSearch( diff, depth - 1, owner((tp + 1)%2) );
        makeMove( (*it).end, (*it).begin );
    }

    return ret;
}

void KornersGameEngineAI::makeNextMove( const QPoint& src, const QPoint& dst )
{
    QPoint src_comp( FIELD_SIZE - src.x() - 1,
                     FIELD_SIZE - src.y() - 1 );
    QPoint dst_comp( FIELD_SIZE - dst.x() - 1,
                     FIELD_SIZE - dst.y() - 1 );
    MoveInfo ret = search(8);   // Count 4 moves froward
    kDebug() << ret.begin.x() << " : " << ret.begin.y() << " ; "
             << ret.end.x() << " : " << ret.end.y();

    int i = m_capPositions.indexOf( ret.begin );
    m_capPositions[i] = ret.end;
    computerMove( ret.begin, ret.end );
}
