
#include "korners_game_engine_network.h"
#include "korners_protocol.h"
#include "korners_socket.h"
#include "korners_server.h"

#include <KDebug>
#include <KLocale>


KornersGameEngineNetwork::KornersGameEngineNetwork( const QString& netName, 
                                                    bool server, 
                                                    int port, 
                                                    const QString& peerIp)
{
    m_nickName = netName;
    m_server = 0;

    m_curGameProto = new KornersGameProtocol(netName, server);

    connect( m_curGameProto, SIGNAL( messageParsed( KornersMessage ) ), 
             SLOT( messageReceived( KornersMessage ) ) );

    m_socket = new KornersSocket( m_curGameProto, this );

    if( server )
    {
        m_server = new KornersServer( m_socket, this );
        m_server->startListen(port);
        connect( m_server, SIGNAL( newConnection() ), SLOT( incomingConnection() ) );
    }
    else
    {
        m_socket->connectTo( peerIp, port );
        connect( m_socket, SIGNAL( connectionFailed() ), SLOT( connectionFailed() ) );
    }

    connect( m_socket, SIGNAL( disconnected() ), SLOT( onDisconnect() ) );
}

KornersGameEngineNetwork::~KornersGameEngineNetwork()
{
    const QByteArray& msg = m_curGameProto->stop();
    m_socket->send(msg);
    m_socket->close();
    if( m_server != 0 )
        m_server->close();
}

void KornersGameEngineNetwork::makeNextMove( const QPoint& src, const QPoint& dst )
{
    const QByteArray& msg  = m_curGameProto->move(src, dst);
    m_socket->send(msg);
}

void KornersGameEngineNetwork::messageReceived( KornersMessage mess )
{
    switch(mess.type)
    {
        case KornersMessage::handshake_s:
            {
                m_peerName = mess.data.front().toString();
                emit( engineMessage( i18n("Player ") + m_peerName + i18n(" connected") ) );
                emit( opponentName(m_peerName) );
                const QByteArray& msg = m_curGameProto->start();
                m_socket->send(msg);
            }
            break;
        case KornersMessage::handshake_c:
            {
                m_peerName = mess.data.front().toString();
                emit( engineMessage( i18n("Player ") + m_peerName + i18n(" connected") ) );
                emit( opponentName(m_peerName) );
            }
            break;
        case KornersMessage::move:
            {
                // Rotate the board
                QPoint src( FIELD_SIZE - mess.data[0].toPoint().x() - 1,
                            FIELD_SIZE - mess.data[0].toPoint().y() - 1 );
                QPoint dst( FIELD_SIZE - mess.data[1].toPoint().x() - 1,
                            FIELD_SIZE - mess.data[1].toPoint().y() - 1 ); 
                computerMove( src, dst );
            }
            break;
        case KornersMessage::regret:
            emit( engineMessage( i18n("Player") + m_peerName + i18n("gives up") ) );
            break;
        default:
            break;
    }
}

void KornersGameEngineNetwork::incomingConnection()
{
    kDebug() << "connected";

    const QByteArray& msg = m_curGameProto->start();
    m_socket->send(msg);
    emit( canPlay() );
}

void KornersGameEngineNetwork::connectionFailed()
{
    emit( engineMessage( i18n("Connection failed") ) );
}

void KornersGameEngineNetwork::onDisconnect()
{
    emit( engineMessage( i18n("Connection lost") ) );
}
